/*
 * Copyright (c) 2020 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef __QSO_H
#define __QSO_H

#include <jeffpc/int.h>
#include <jeffpc/val.h>
#include <jeffpc/uuid.h>

enum time_parts {
	TIME_PART_DATE =	0x01, /* year, month, day */
	TIME_PART_TIME_HM =	0x02, /* hours, minutes */
	TIME_PART_TIME_S =	0x04, /* seconds */
#define TIME_PART_TIME_HMS	(TIME_PART_TIME_HM | TIME_PART_TIME_S)
};

struct time {
	uint16_t year;	/* 0-65535 */
	uint8_t month;	/* 1-12 */
	uint8_t day;	/* 1-31 */
	uint8_t hour;	/* 0-23 */
	uint8_t min;	/* 0-59 */
	uint8_t sec;	/* 0-60 */
	enum time_parts have:8;
};

struct qso {
	struct xuuid uuid;
	char uuid_str[XUUID_PRINTABLE_STRING_LENGTH];

	struct {
		struct str *call;
		struct str *grid;
		uint64_t power; /* mW */
	} me;
	struct time start;
	struct time end;
	struct str *call;
	uint64_t freq; /* Hz */
	struct str *mode;
	struct str *rsts;
	struct str *rstr;
	struct str *name;
	struct str *qth;
	struct str *grid;
	struct str *country;
	struct str *state;
	struct str *county;
	struct str *comment;
	struct str *continent;
	struct str *dxcc;
	uint64_t dxcc_code;
	uint64_t itu;
	uint64_t waz;
};

extern int qso_save(const struct qso *qso, int data_dir);
extern void qso_reset(struct qso *qso, bool full);

/* Set the various parts of a QSO.  All passed in struct str * are consumed. */
extern void qso_set_call(struct qso *qso, bool me, char *call);
extern void qso_set_freq(struct qso *qso, char *str);
extern void qso_set_grid(struct qso *qso, bool me, char *grid);
extern void qso_set_power(struct qso *qso, uint64_t milliwatts);
extern void qso_set_power_user(struct qso *qso, char *power);
extern void qso_set_datetime(struct qso *qso, bool start, char *str);
extern void qso_set_mode(struct qso *qso, char *mode);
extern void qso_set_rst(struct qso *qso, bool recv, char *rst);
extern void qso_set_name(struct qso *qso, struct str *name);
extern void qso_set_qth(struct qso *qso, struct str *qth);
extern void qso_set_country(struct qso *qso, struct str *country);
extern void qso_set_state(struct qso *qso, struct str *state);
extern void qso_set_county(struct qso *qso, struct str *county);
extern void qso_set_comment(struct qso *qso, struct str *comment);
extern void qso_set_zone_continent(struct qso *qso, const char *zone);
extern void qso_set_zone_dxcc(struct qso *qso, struct str *zone, uint64_t code);
extern void qso_set_zone_dxcc_user(struct qso *qso, char *zone, uint64_t code);
extern void qso_set_zone_itu(struct qso *qso, uint64_t zone);
extern void qso_set_zone_itu_user(struct qso *qso, char *str);
extern void qso_set_zone_waz(struct qso *qso, uint64_t zone);
extern void qso_set_zone_waz_user(struct qso *qso, char *str);

#endif
